<?php

header('Content-Type: application/json');

$protocol = "https";
include_once($_SERVER['DOCUMENT_ROOT'] . '/inc/origin.class.php');
include_once($_SERVER['DOCUMENT_ROOT'] . '/inc/config.php');

originHelper\origin::check($config['allowed-origins']);

if (isset($_GET['protocol'])) {
  $protocol = $_GET['protocol'];
}
if (!($protocol == "https" || $protocol == "smtp" || $protocol == "ftp")) {
  dieWithMsg("the script only supports connection types of https, smtp or ftp");
}
$port = 0;
if (isset($_GET['port'])) {
  $port = intval($_GET['port']);
}
if (!is_int($port) || $port <= 0 || $port >= 65535) {
  dieWithMsg("port must be numeric");
}
$domain = "";
if (isset($_GET['domain'])) {
  $domain = $_GET['domain'];
}
if (!preg_match('/^(((?!-))(xn--|_{1,1})?[a-z0-9-]{0,61}[a-z0-9]{1,1}\.){1,}(xn--)?([a-z0-9\-]{1,61}|[a-z0-9-]{1,30}\.[a-z]{2,})$/', $domain)) {
  dieWithMsg("invalid domain provided.");
}
if (preg_match('/^.*\.(local(domain)?|hole)$/', $domain)) {
  dieWithMsg("domains with .local or .localdomain are not allowed!");
}
if (!fixed_gethostbyname($domain)) {
  dieWithMsg("could not resolve host $domain (no ip-addresses allowed due to ssl-certificate cn and security)");
}
$options = 0;
if (isset($_GET['options'])) {
  $options = abs(intval($_GET['options']));
}
if (!is_int($options) || $options < 0 || $options >= 64) {
  $options = 0;
}
$extparams = "";
if ($options > 0) {
  if ($options & 1) {
    $extparams .= "-showcerts ";
  }
  if ($options & 2) {
    $extparams .= "-no_tls1 ";
  }
  if ($options & 4) {
    $extparams .= "-no_tls1_1 ";
  }
  if ($options & 8) {
    $extparams .= "-no_tls1_2 ";
  }
  if ($options & 16) {
    $extparams .= "-no_tls1_3 ";
  }
  if ($options & 32) {
    $extparams .= "-tlsextdebug ";
  }
}
$starttls = "";
if ($protocol == "smtp" || $protocol == "ftp") {
  $starttls = "-starttls $protocol ";
}
$cmd = "/usr/bin/openssl s_client -CApath /etc/ssl/certs/ -connect " . escapeshellarg("$domain:$port") . " $starttls$extparams < " . __DIR__ . "/quit.txt 2>&1";

$ret = array();
$ret[] = array('type' => 'params', 'content' => $extparams);

$rettext = exec_timeout($cmd, 5);

$ret = array_merge($ret, splitWithRegex($rettext, '/(^-----BEGIN CERTIFICATE-----\n[a-zA-Z0-9\/\n+=]*\n-----END CERTIFICATE-----$)/m', 'cert', function ($deep1) {
  return splitWithRegex($deep1, '/(^Verification: OK$)/m', 'success', function ($deep2) {
    return splitWithRegex($deep2, '/(^Verification\s?:?\s?[Ee]rror.*$)/m', 'error', 'text');
  });
}));

echo json_encode($ret);

function fixed_gethostbyname(string $host)
{
  $ip = gethostbyname($host);
  if ($ip != $host) {
    return $ip;
  } else return false;
}

// source: https://blog.dubbelboer.com/2012/08/24/execute-with-timeout.html
/**
 * Execute a command and return it's output. Either wait until the command exits or the timeout has expired.
 *
 * @param string $cmd     Command to execute.
 * @param int $timeout Timeout in seconds.
 * @return string Output of the command.
 * @throws \Exception
 */
function exec_timeout(string $cmd, int $timeout): string
{
  // File descriptors passed to the process.
  $descriptors = array(
    0 => array('pipe', 'r'),  // stdin
    1 => array('pipe', 'w'),  // stdout
    2 => array('pipe', 'w')   // stderr
  );

  // Start the process.
  $process = proc_open('exec ' . $cmd, $descriptors, $pipes);

  if (!is_resource($process)) {
    throw new \Exception('Could not execute process');
  }

  // Set the stdout stream to non-blocking.
  stream_set_blocking($pipes[1], 0);

  // Set the stderr stream to non-blocking.
  stream_set_blocking($pipes[2], 0);

  // Turn the timeout into microseconds.
  $timeout = $timeout * 1000000;

  // Output buffer.
  $buffer = '';

  $normalexit = false;
  // While we have time to wait.
  while ($timeout > 0) {
    $start = microtime(true);

    // Wait until we have output or the timer expired.
    $read  = array($pipes[1]);
    $other = array();
    stream_select($read, $other, $other, 0, (int)$timeout);

    // Get the status of the process.
    // Do this before we read from the stream,
    // this way we can't lose the last bit of output if the process dies between these functions.
    $status = proc_get_status($process);

    // Read the contents from the buffer.
    // This function will always return immediately as the stream is non-blocking.
    $buffer .= stream_get_contents($pipes[1]);

    if (!$status['running']) {
      $normalexit = true;
      // Break from this loop if the process exited before the timeout.
      break;
    }

    // Subtract the number of microseconds that we waited.
    $timeout -= (microtime(true) - $start) * 1000000;
  }

  // Check if there were any errors.
  $errors = stream_get_contents($pipes[2]);

  if (!empty($errors)) {
    throw new \Exception($errors);
  }

  if (!$normalexit) {
    $buffer .= "\nTimeout!";
  }
  // Kill the process in case the timeout expired and it's still running.
  // If the process already exited this won't do anything.
  proc_terminate($process, 9);

  // Close all streams.
  fclose($pipes[0]);
  fclose($pipes[1]);
  fclose($pipes[2]);

  proc_close($process);

  return $buffer;
}

function splitWithRegex(string $text, string $regex, string $matchType, string|callable $otherType): array
{
  $return = array();
  $startIdx = 0;
  if (preg_match_all($regex, $text, $matches, PREG_OFFSET_CAPTURE | PREG_SET_ORDER) !== false) {
    foreach ($matches as $match) {
      $matchPos = $match[1][1];
      $matchText = $match[1][0];
      if ($startIdx < $matchPos) {
        if (is_callable($otherType)) {
          $return = array_merge($return, $otherType(substr($text, $startIdx, $matchPos - $startIdx)));
        } else {
          $return[] = array('type' => $otherType, 'content' => substr($text, $startIdx, $matchPos - $startIdx));
        }
      }
      $return[] = array('type' => $matchType, 'content' => $matchText);
      $startIdx = $matchPos + strlen($matchText);
    }
  }
  if ($startIdx < strlen($text)) {
    if (is_callable($otherType)) {
      $return = array_merge($return, $otherType(substr($text, $startIdx)));
    } else {
      $return[] = array('type' => $otherType, 'content' => substr($text, $startIdx));
    }
  }
  return $return;
}

function dieWithMsg(string $message)
{
  http_response_code(400);
  echo json_encode(array('error' => $message));
  die;
}
