<?php
include_once($_SERVER['DOCUMENT_ROOT'].'/inc/origin.class.php');
include_once($_SERVER['DOCUMENT_ROOT'].'/inc/config.php');
include_once($_SERVER['DOCUMENT_ROOT'].'/inc/funcs.php');

originHelper\origin::check($config['allowed-origins']);

// all the stations to combine
$stations = $config['transport-stations'];
// number of connections to fetch per station
$numPerStation = $config['transport-num-per-station'];
// number of connections (max) in final output
$numReturn = $config['transport-num-results'];


setlocale(LC_TIME,"de_DE");

class JsonDateTime extends \DateTime implements \JsonSerializable
{
    public function jsonSerialize() : mixed
    {
       return $this->format("c");
    }
}

class FahrplanEntry {
  public $Departure;
  public $Delay = '';
  public $Name = '';
  public $To = '';
  public $Platform = '';
}

$stationArray = [];

function getStationArray($station, $num) {
  $ret = [];
  $stationurl="https://timetable.search.ch/api/stationboard.json?stop=".urlencode($station)."&show_delays=1&show_tracks=1&limit=$num";
  try {
    $stationboard = json_decode(apiFuncs\xsget($stationurl), true);
    foreach($stationboard["connections"] as $itm) {
      $fe = new FahrplanEntry();
      if(isset($itm["dep_delay"])) {
        if($itm["dep_delay"] !== "+0" && $itm["dep_delay"] !== "+1") {
          $fe->Delay = $itm["dep_delay"];
        }
      }
      $fe->Departure = new JsonDateTime($itm["time"]);
      $fe->Name = isset($itm["*G"]) ? $itm["*G"] : '';
      $fe->Name .= isset($itm["*L"]) ? $itm["*L"] : '';
      if(isset($itm["terminal"]["name"])) {
        if($itm["terminal"]["name"] == $station) {
        $fe->To = "St. Gallen";
        } else {
          $fe->To = $itm["terminal"]["name"];
        }
      }
      $fe->Platform = isset($itm["track"]) ? $itm["track"] : '';
      $ret[] = $fe;
    }
  } catch (Exception $ex) {
    $fe = new FahrplanEntry();
    $fe->Departure = new JsonDateTime();
    $fe->Name = "Fehler!";
    $fe->To = $ex->getMessage();
    $ret[] = $fe;
  }
  return $ret;
}

foreach($stations as $station) {
  $stationArray = array_merge($stationArray, getStationArray($station,$numPerStation));
}

usort($stationArray, function($a, $b) {
  if($a->Departure == $b->Departure) {
    return 0;
  }
  return $a->Departure < $b->Departure ? -1 : 1;
});

$stationArray = array_slice($stationArray,0,$numReturn);

header('Content-Type: application/json');
echo json_encode($stationArray);
