<?php
include_once($_SERVER['DOCUMENT_ROOT'] . '/inc/origin.class.php');
include_once($_SERVER['DOCUMENT_ROOT'] . '/inc/config.php');
include_once($_SERVER['DOCUMENT_ROOT'] . '/inc/funcs.php');

originHelper\origin::check($config['allowed-origins']);

setlocale(LC_TIME, "de_DE");

class WeatherEntry
{
  public $Date;
  public $DayName;
  public $Icon = '';
  public $Main = '';
  public $TempLow = 0.0;
  public $TempHigh = 0.0;
  public $Wind;
  public $Humidity;
  public $PercRain;
  public $PercSnow;
}

function parseWeather($list)
{
  $ret = [];
  $days = [];
  $perday = [];
  $currDay = '';
  foreach ($list as $entry) {
    $curr = new WeatherEntry();
    $curr->Date = new DateTime($entry->dt_txt);
    $curr->DayName = apiFuncs\formatAsDay($curr->Date->format('N'));
    $curr->Icon = $entry->weather[0]->id;
    $curr->Main = $entry->weather[0]->main;
    $curr->TempLow = round($entry->main->temp_min, 1);
    $curr->TempHigh = round($entry->main->temp_max, 1);
    $curr->Wind = $entry->wind->speed;
    $curr->Humidity = $entry->main->humidity;
    //$curr->PercRain = $entry->rain->{'3h'};
    //$curr->PercSnow = $entry->snow->{'3h'};

    $day = $curr->Date->format('d.n');
    if ($currDay == '') {
      $currDay = $day;
    }
    if ($day != $currDay) {
      if (count($perday) > 0) {
        $ret[] = ['key' => $currDay, 'list' => $perday];
      }
      $currDay = $day;
      $perday = [];
    }
    $perday[] = $curr;
  }
  if ($currDay != "") {
    $ret[] = ['key' => $currDay, 'list' => $perday];
  }

  return $ret;
}


$weatherCache = "cache/.weather.cache";
$weatherUrl = "https://api.openweathermap.org/data/2.5/forecast";
$weatherUrl .= "?q=" . $config['weather-location'] . "&units=metric&APPID=" . $config['weather-api']; //&cnt=16
try {
  $weatherContent = json_decode(apiFuncs\cache_contents($weatherUrl, $weatherCache, 10800, false));  // 10800 = 3 Stunden
} catch (Exception $ex) {
  die("<h2>error fetching weather data! <small>(" . $ex->getMessage() . ")</small></h2>");
}
$iconClass = "wi wi-owm-";

$forecast = $weatherContent->list;
$arr = parseWeather($forecast);

// today
$tmpl = '<div class="col wcol"><div class="today"><div class="wicon"><i class="' . $iconClass . $arr[0]['list'][0]->Icon . '" title="' . $arr[0]['list'][0]->Main . '"></i></div>';
$tmpl .= '<div class="maintemp">' . $arr[0]['list'][0]->TempLow . '&#92;&nbsp;<span class="bigger">' . $arr[0]['list'][0]->TempHigh . '&deg;C</span></div>';
$tmpl .= '</div></div>';

// rest of forecast
for ($i = 1; $i < count($arr) - 1; $i++) {
  $tdy = $arr[$i]['list'];
  $night = '';
  $day = '';
  $cnt = count($tdy);
  switch ($cnt) {
    case 1:
      $night = $tdy[0];
      $day = $tdy[0];
      break;
    case 2:
      $night = $tdy[0];
      $day = $tdy[1];
      break;
    case 3:
      $night = $tdy[0];
      $day = $tdy[2];
      break;
    case 4:
      $night = $tdy[0];
      $day = $tdy[3];
      break;
    default:
      $night = $tdy[0];
      $day = $tdy[4];
      break;
  }
  $tmpl .= '<div class="col wcol"><span class="bigger">' . $day->DayName . '</span><div class="wicon"><i class="' . $iconClass . $day->Icon . '" title="' . $day->Main . '"></i></div>';
  $tmpl .= $night->TempLow . '&#92;&nbsp;<span class="bigger">' . $day->TempHigh . '&deg;C</span></div>';
}
echo $tmpl;
