<?php

include_once($_SERVER['DOCUMENT_ROOT'] . '/inc/origin.class.php');
include_once($_SERVER['DOCUMENT_ROOT'] . '/inc/config.php');
include_once($_SERVER['DOCUMENT_ROOT'] . '/inc/funcs.php');

originHelper\origin::check($config['allowed-origins']);

header('Content-Type: application/json');

setlocale(LC_TIME, "de_DE");

class JsonDateTime extends \DateTime implements \JsonSerializable
{
	public function jsonSerialize(): mixed
	{
		return $this->format("c");
	}
}

class WeatherEntry
{
	public $Date;
	public $Icon = '';
	public $TempLow = 0.0;
	public $TempHigh = 0.0;
	public $RainsHigh = 0.0;
	public $Temps = [];
	public $Rains = [];
}

function translateIconClass($class)
{
	if ($class === null) {
		return null;
	}
	$class = strtolower($class);
	$postfix = null;
	if (str_ends_with($class, '_day')) {
		$postfix = 'day';
		$class = substr($class, 0, strlen($class) - 4);
	} elseif (str_ends_with($class, '_night')) {
		$postfix = 'night';
		$class = substr($class, 0, strlen($class) - 6);
	}
	if ($class === 'clearsky') {
		if ($postfix === 'night') {
			return 'wi-night-clear';
		}
		return 'wi-day-sunny';
	}
	$src = array(
		'cloudy' => 'wi-cloudy',
		'fair' => 'wi-POSTFIX-cloudy',
		'fog' => 'wi-POSTFIX-fog',
		'heavyrain' => 'wi-rain',
		'heavyrainandthunder' => 'wi-POSTFIX-storm-showers',
		'heavyrainshowers' => 'wi-POSTFIX-rain',
		'heavyrainshowersandthunder' => 'wi-thunderstorm',
		'heavysleet' => 'wi-POSTFIX-sleet',
		'heavysleetandthunder' => 'wi-POSTFIX-sleet-storm',
		'heavysleetshowers' => 'wi-POSTFIX-sleet',
		'heavysleetshowersandthunder' => 'wi-POSTFIX-sleet-storm',
		'heavysnow' => 'wi-POSTFIX-snow',
		'heavysnowandthunder' => 'wi-POSTFIX-snow-thunderstorm',
		'heavysnowshowers' => 'wi-POSTFIX-snow',
		'heavysnowshowersandthunder' => 'wi-POSTFIX-snow-thunderstorm',
		'lightrain' => 'wi-POSTFIX-rain',
		'lightrainandthunder' => 'wi-POSTFIX-thunderstorm',
		'lightrainshowers' => 'wi-POSTFIX-rain',
		'lightrainshowersandthunder' => 'wi-POSTFIX-thunderstorm',
		'lightsleet' => 'wi-POSTFIX-rain-mix',
		'lightsleetandthunder' => 'wi-POSTFIX-thunderstorm',
		'lightsleetshowers' => 'wi-POSTFIX-rain-mix',
		'lightsnow' => 'wi-POSTFIX-snow',
		'lightsnowandthunder' => 'wi-POSTFIX-snow-thunderstorm',
		'lightsnowshowers' => 'wi-POSTFIX-snow',
		'lightssleetshowersandthunder' => 'wi-POSTFIX-snow-thunderstorm',
		'lightssnowshowersandthunder' => 'wi-POSTFIX-snow-thunderstorm',
		'partlycloudy' => 'wi-POSTFIX-cloudy',
		'rain' => 'wi-POSTFIX-rain',
		'rainandthunder' => 'wi-POSTFIX-thunderstorm',
		'rainshowers' => 'wi-POSTFIX-rain',
		'rainshowersandthunder' => 'wi-POSTFIX-thunderstorm',
		'sleet' => 'wi-POSTFIX-rain-mix',
		'sleetandthunder' => 'wi-POSTFIX-thunderstorm',
		'sleetshowers' => 'wi-POSTFIX-rain-mix',
		'sleetshowersandthunder' => 'wi-POSTFIX-thunderstorm',
		'snow' => 'wi-POSTFIX-snow',
		'snowandthunder' => 'wi-POSTFIX-snow-thunderstorm',
		'snowshowers' => 'wi-POSTFIX-snow',
		'snowshowersandthunder' => 'wi-POSTFIX-snow-thunderstorm',
	);
	if (isset($src[$class])) {
		$ret = $src[$class];
		if (str_contains($ret, 'POSTFIX')) {
			$ret = str_replace('POSTFIX', ($postfix ?? 'day'), $ret);
		}
		return $ret;
	} else {
		return 'wi-na ' . $class;
	}
}

function parseWeather($list)
{
	$maxDays = 5;
	$numDay = 1;
	$ret = [];
	$perday = [];
	$currDay = '';
	foreach ($list as $entry) {
		$curr = new WeatherEntry();
		$curr->Date = new JsonDateTime($entry->time);
		$curr->DayName = apiFuncs\formatAsDay($curr->Date->format('N'));
		$curr->Icon = translateIconClass($entry->data->next_12_hours?->summary?->symbol_code ?? null);
		$curr->TempHigh = $entry->data->instant->details->air_temperature;
		$curr->TempLow = $entry->data->next_1_hours?->details?->precipitation_amount ?? null;

		$day = $curr->Date->format('d.n');
		if ($currDay == '') {
			$currDay = $day;
		}
		if ($day != $currDay) {
			if (count($perday) > 0) {
				$ret[] = ['day' => $currDay, 'data' => summarizeDay($perday)];
				$numDay++;
				if ($numDay > $maxDays) {
					$currDay = "";
					$perday = [];
					break;
				}
			}
			$currDay = $day;
			$perday = [];
		}
		$perday[] = $curr;
	}
	if ($currDay != "") {
		$ret[] = ['day' => $currDay, 'data' => summarizeDay($perday)];
	}

	return $ret;
}

function summarizeDay($dayArray)
{
	// find min/max temp per day, returns one summary entry
	$ret = new WeatherEntry();
	$temps = [];
	$rains = [];
	$first = $dayArray[0];
	$ret->Date = $first->Date;
	$ret->DayName = $first->DayName;
	$ret->Icon = $first->Icon;
	$ret->TempLow = $first->TempHigh; // steht alles in TempHigh.
	$ret->TempHigh = $first->TempHigh;
	$ret->RainsHigh = $first->TempLow; // rain steht in TempLow...
	foreach ($dayArray as $entry) {
		if ($entry->TempHigh < $ret->TempLow) {
			$ret->TempLow = $entry->TempHigh;
		}
		if ($entry->TempHigh > $ret->TempHigh) {
			$ret->TempHigh = $entry->TempHigh;
		}
		if ($entry->TempLow ?? 0.0 > $ret->RainsHigh ?? 0.0) {
			$ret->RainsHigh = $entry->TempLow;
		}
		$temps[$entry->Date->format('H:i')] = $entry->TempHigh;
		$rains[$entry->Date->format('H:i')] = $entry->TempLow;
		if ($ret->Icon == null && $entry->Icon != null) {
			$ret->Icon = translateIconClass($entry->Icon);
		}
	}
	$ret->Temps = $temps;
	$ret->Rains = $rains;
	return $ret;
}

$weatherCache = "cache/.weather2.cache";
$weatherUrl = "https://api.met.no/weatherapi/locationforecast/2.0/compact";
$weatherUrl .= "?altitude=" . $config['weather-alt'] . "&lat=" . $config['weather-lat'] . "&lon=" . $config['weather-lon'];
try {
	$weatherContent = json_decode(apiFuncs\cache_contents($weatherUrl, $weatherCache, 10800, true));  // 10800 = 3 Stunden
} catch (Exception $ex) {
	// header('Content-Type: application/json');
	die(json_encode(array('err' => "error fetching weather data! (" . $ex->getMessage() . ")")));
}

$forecast = $weatherContent->properties->timeseries;
$arr = parseWeather($forecast);

echo json_encode($arr);
