/* global $, Vue, moment */
var d = new Date();
var lastCarousel = new Date();

var baseUrl = "https://api.mayer.life/dashboard/";

var nightStartHour = 0;
var nightStopHour = 5;
var nightInterval = 1800000; // 30 minutes
var dayInterval = 120000; // 2 minutes
var fourHinterval = 14400000; // 4 hours
var clockInterval = 20000; // 20 seconds
var isClearing = false;
var clearerInterval = -1;

function hasToUpdate() {
  var rightnow = new Date();
  var sinceCarousel = (rightnow.getTime() - lastCarousel.getTime()) / 1000;
  if (sinceCarousel > 120) {
    lastCarousel = rightnow;
    return true;
  } else {
    return false;
  }
}

var transportVue = new Vue({
  data: {
    fahrplanEntries: [],
    timer: '',
    disableTimer: '',
    lastUpdate: moment().format('HH:mm:ss'),
    lastTimeout: '',
    debugVisible: false,
    fahrplanActive: false
  },
  created: function () {
    if (this.fahrplanActive) {
      this.fetchTransportList();
    }
  },
  methods: {
    fetchTransportList: function () {
      var self = this;
      //console.log("fetchTransportList started.");
      $.getJSON(baseUrl + 'api/transport.php')
        .done(function (entries) {
          self.fahrplanEntries = entries;

          self.lastUpdate = moment().format('HH:mm:ss');

          var newTimeout = dayInterval;
          var h = new Date();
          h = h.getHours();
          if (h >= nightStartHour && h <= nightStopHour) // night interval
          {
            newTimeout = nightInterval;
          }
          if (entries.length > 0) {
            var nextDeparture = moment(entries[0].Departure);
            var timeToNextDep = nextDeparture.diff(moment());
            timeToNextDep += 60000; // plus 1 minute
            if (timeToNextDep > newTimeout) {
              newTimeout = timeToNextDep;
            }
          }
          self.timer = setTimeout(self.fetchTransportList, newTimeout);
          /* debug information start */
          var tempTimeout = Math.round(newTimeout / 1000);
          if (tempTimeout > 60) {
            var mins = Math.floor(tempTimeout / 60);
            var rest = tempTimeout - (60 * mins);
            tempTimeout = mins + ':' + rest + 'min';
          } else {
            tempTimeout = tempTimeout + 's';
          }
          self.lastTimeout = tempTimeout;
          /* debug information end */
        }).fail(function () {
          if (this.fahrplanActive) {
            this.fahrplanEntries = [
              { "Departure": undefined, "Name": "Fehler beim Laden der Daten! WiFi neu verbinden." },
              { "Departure": undefined, "Name": "" },
              { "Departure": undefined, "Name": "" },
              { "Departure": undefined, "Name": "" },
              { "Departure": undefined, "Name": "" },
              { "Departure": undefined, "Name": "" },
            ];
          }
          self.lastTimeout = 'error, waiting 2min...';
          self.timer = setTimeout(self.fetchTransportList, dayInterval);
        });
    },
    cancelAutoUpdate: function () { clearInterval(this.timer); },
    setColspan: function (data) {
      if (data == undefined || data == '') {
        return 2;
      }
      return 1;
    },
    toggleFahrplan: function () {
      this.fahrplanActive = !this.fahrplanActive;
      if (this.fahrplanActive) {
        this.fahrplanEntries = [
          { "Departure": new moment(), "Name": "Lade Daten..." },
          { "Departure": undefined, "Name": "" },
          { "Departure": undefined, "Name": "" },
          { "Departure": undefined, "Name": "" },
          { "Departure": undefined, "Name": "" },
          { "Departure": undefined, "Name": "" },
        ];
        this.fetchTransportList();
        this.disableTimer = setTimeout(this.toggleFahrplan, 600000); // disable fahrplan after ten minutes
      } else {
        this.cancelAutoUpdate();
        if (this.disableTimer) {
          clearInterval(this.disableTimer);
        }
      }
    },
    toggleDebug: function () {
      this.debugVisible = !this.debugVisible;
    }
  },
  filters: {
    formatDate: function (date) {
      return moment(date).format('HH:mm');
    }
  },
  beforeDestroy: function () { clearInterval(this.timer); },
  el: '#stationboard'
});

var regioterreVue = new Vue({
  data: {
    texts: [],
    alltext: '',
    dow: 0,
    timer: ''
  },
  created: function () {
    this.updateTexts();
  },
  methods: {
    updateTexts: function () {
      var dat = new Date();
      this.dow = dat.getDay();
      if (this.isMonOrTue()) { // Montag, Dienstag
        var self = this;
        $.getJSON(baseUrl + 'api/regioterre.php')
          .done(function (entries) {
            self.texts = entries.texts;
            self.alltext = entries.alltext;
          })
          .fail(function (err) {
            self.alltext = "Error " + err;
          });
      }
      this.timer = setTimeout(this.updateTexts, fourHinterval);
    },
    isTableDisplay: function () {
      return this.isMonOrTue() && (this.texts.length > 0 || this.alltext != "");
    },
    isMonOrTue: function () {
      return (this.dow == 1 || this.dow == 2);
    },
    cancelAutoUpdate: function () { clearInterval(this.timer); }
  },
  beforeDestroy: function () { this.cancelAutoUpdate(); },
  el: '#regioterre'
});

var waescheVue = new Vue({
  data: {
    laundrytext: '',
    dow: 0,
    timer: '',
    laundrydays: {
      2: 'Wäsche?',     // Dienstag
      3: 'Wäsche holen'  // Mittwoch
    }
  },
  created: function () {
    this.updateLaundryText();
  },
  methods: {
    updateLaundryText: function () {
      var dat = new Date();
      this.dow = dat.getDay();
      if (!!this.laundrydays[this.dow]) {
        this.laundrytext = this.laundrydays[this.dow];
      } else {
        this.laundrytext = "";
      }

      this.timer = setTimeout(this.updateLaundryText, fourHinterval);
    },
    isLaundryDay: function () {
      return !!this.laundrydays[this.dow];
    },
    cancelAutoUpdate: function () { clearInterval(this.timer); }
  },
  beforeDestroy: function () { this.cancelAutoUpdate(); },
  el: '#waesche'
});

var clockVue = new Vue({
  data: {
    date: '',
    time: '',
    timer: ''
  },
  created: function () {
    this.updateTime();
  },
  methods: {
    updateTime: function () {
      var dat = new Date();
      this.time = this.lzero(dat.getHours(), 2) + ':' + this.lzero(dat.getMinutes(), 2);
      //this.date = this.lzero(dat.getDate(),2) + '.' + this.lzero(dat.getMonth()+1,2) + '.';

      this.timer = setTimeout(this.updateTime, clockInterval);
    },
    lzero: function (str, len) {
      str += '';
      while (str.length < len) {
        str = "0" + str;
      }
      return str;
    },
    cancelAutoUpdate: function () { clearInterval(this.timer); }
  },
  beforeDestroy: function () { this.cancelAutoUpdate(); },
  el: '#clock'
});

var weatherVue = new Vue({
  data: {
    today: {},
    forecast: {},
    timer: ''
  },
  created: function () {
    this.updateWeather();
  },
  methods: {
    updateWeather: function () {
      var self = this;
      $.getJSON(baseUrl + 'api/weather2.php')
        .done(function (forecast) {
          self.today = forecast.shift();
          self.forecast = forecast;
        })
        .fail(function (err) {
          self.alltext = "Error " + err;
        });
      this.timer = setTimeout(this.updateWeather, fourHinterval);
    },
    cancelAutoUpdate: function () { clearInterval(this.timer); }
  },
  beforeDestroy: function () { this.cancelAutoUpdate(); },
  el: '#weather'
});


function clearerWipe() {
  if (!isClearing) {
    isClearing = true;
    var clearer = document.getElementById('clearer');
    clearer.classList.add('clear1');
    setTimeout(function () {
      clearer.classList.remove('clear1');
      clearer.classList.add('clear2');
      setTimeout(function () {
        clearer.classList.remove('clear2');
        isClearing = false;
      }, 300);
    }, 300);
  }
}

function turnscreen() {
  var wrap = document.getElementById("vwrap");
  if (wrap) {
    if (wrap.classList.contains('vertical-wrapper')) {
      wrap.classList.remove('vertical-wrapper');
    } else {
      wrap.classList.add('vertical-wrapper');
    }
  }
}

document.addEventListener("DOMContentLoaded", function (event) {
  clearerInterval = setInterval(clearerWipe, 3600000); // every 1 hour

  // online/offline detection (display refresh button inversed when offline)
  var setOnline = function (online) {
    var reloadBtn = document.getElementById('reloader');
    if (online) {
      reloadBtn.classList.remove('offline');
    } else {
      reloadBtn.classList.add('offline');
    }
  }
  var onlineTimer = '';
  var setOnlineDelayed = function (online) {
    if (onlineTimer) {
      window.clearTimeout(onlineTimer);
    }
    onlineTimer = window.setTimeout(setOnline(online), 200);
  }

  if (window.navigator.online !== undefined && !window.navigator.online) {
    var reloadBtn = document.getElementById('reloader');
    reloadBtn.classList.add('offline');
  }
  window.addEventListener('online', function () { setOnlineDelayed(true); });
  window.addEventListener('offline', function () { setOnlineDelayed(false); });
});
