<?php

namespace ApiMayerLife;

use Exception;

class Ntfy
{
  private string $topic;
  private ?string $token;
  private string $serverUrl;
  private const DEFAULT_URL = 'https://ntfy.sh';

  public function __construct(?string $topic = null, ?string $token = null, ?string $serverUrl = null)
  {
    // topic: set priorities: 1. $topic, 2. environment NTFY_TOPIC, 3. throw exception
    $this->topic = $this->getValueFromEnv($topic, 'NTFY_TOPIC');
    if ($this->topic === null) {
      throw new Exception("Ntfy.sh topic must be set!");
    }
    // token: set priorities: 1. $token, 2. environment NTFY_TOKEN, 3. unset.
    $this->token = $this->getValueFromEnv($token, 'NTFY_TOKEN');

    // url: set priorities: 1. $serverUrl parameter, 2. environment NTFY_URL, 3. default url.
    $url = $this->getValueFromEnv($serverUrl, 'NTFY_URL');
    $this->serverUrl = $url ?? self::DEFAULT_URL;
  }

  public function publish(
    string $title,
    string $text,
    ?string $tags = null,
    ?string $url = null,
    ?string $urlTitle = null
  ): bool {
    $headers = ["Content-Type: text/markdown"];
    if ($title) {
      $headers[] = "Title: $title";
    }
    if ($this->token) {
      $headers[] = "Authorization: Bearer $this->token";
    }
    if ($tags) {
      $headers[] = "Tags: $tags";
    }
    if ($url) {
      $urlTitle = $urlTitle ?? "Open url";
      $headers[] = "Actions: view, $urlTitle, $url";
    }
    $header = join("\r\n", $headers);
    $ret = file_get_contents("$this->serverUrl/$this->topic", false, stream_context_create([
      'http' => [
        'method' => 'POST',
        'header' => $header,
        'content' => $text
      ]
    ]));
    return $ret ? true : false;
  }

  private function getValueFromEnv(?string $value, string $envName): ?string
  {
    if (!$value && isset($_ENV[$envName])) {
      $envValue = $_ENV[$envName];
      if ($envValue) {
        return $envValue;
      }
    }
    return $value;
  }
}
