<?php
namespace gmaChannels;

class Channel {
  public $Name;
  public $ID;
  public $Url;
  public $Logo;
  public $Group;
  public function __construct($name = null, $url = null, $group = null, $id = null, $logo = null)
  {
    if ($name != null) {
      $this->Name = $name;
      $this->Url = $url;
    }
    $this->ID = $id;
    $this->Logo = $logo;
    $this->Group = $group;
  }
  
  public function ToString($num = null) {
    $ret = "#EXTINF:0";
    if($this->Logo != null) {
      $ret .= ' tvg-logo="'.$this->Logo.'"';
    }
    if($num != null) {
      $ret .= ' tvg-chno="'.$num.'"';
    }
    if($this->ID != null) {
      $ret .= ' tvg-name="'.$this->ID.'"';
    }
    if($this->Group != null) {
      $ret .= ' group-title="'.$this->Group.'"';
    }
    return $ret.", ".$this->Name."\n".$this->Url."\n";
  }
}

class ChannelParser {
  protected $firstList;
  protected $restList;
  
  protected $favourites;
  protected $filterGroups;
  protected $debug;
  
  public function __construct(array $favs, array $groups, $debug) {
    $this->firstList = array();
    $this->restList = array();
    $this->favourites = $favs;
    $this->filterGroups = $groups;
    $this->debug = $debug;
  }
  
  public function Parse($in) {
    $l1 = null;
    foreach(preg_split("/((\r?\n)|(\r\n?))/", $in) as $line){
      if(!preg_match("/#EXTM3.*/",$line)) {
        if(preg_match('/#EXTINF:-?\\d\\s+([^,]+),\\s?(.*)/',$line, $matches)) {
          //$matches index: 1=Zusatzinfos, 2=name
          $l1 = $matches;
        } else {
          $logo = null;
          $id = null;
          $group = null;
          if(preg_match('/.*?tvg-logo=["]([^"]+)["].*?/',$l1[1],$temp)) {
            $logo = $temp[1];
          }
          if(preg_match('/.*?tvg-name=["]([^"]+)["].*?/',$l1[1],$temp)) {
            $id = $temp[1];
          }
          if(preg_match('/.*?group-title=["]([^"]+)["].*?/',$l1[1],$temp)) {
            $group = $temp[1];
          }
          $add = true;
          if($group != null && ! \in_array($group, $this->filterGroups)) {
            $add = false;
          }
          if($add) {
            $this->Add(new Channel($l1[2], $line, $group, $id, $logo));
          }
        }
      }
    }
    
    // sort first list to match favourites
    $firstsorted = array();
    foreach($this->firstList as $chnl) {
      $key = array_search($chnl->Name, $this->favourites);
      if($key >= 0) {
        $firstsorted[$key] = $chnl;
      }
    }
    ksort($firstsorted);
    $this->firstList = $firstsorted;
  }

  protected function Add(Channel $c) {
    if(\in_array($c->Name,$this->favourites)) {
      $this->firstList[] = $c;
    } else {
      $this->restList[] = $c;
    }
    
  }
  
  protected function GetChannels() {
    return \array_merge($this->firstList, $this->restList);
  }
  
  public function GetList() {
    $ret = "#EXTM3U\n";
    $num = 1;
    foreach($this->GetChannels() as $item) {
      $ret .= $item->ToString($num);
      $num++;
    }
    return $ret;
  }
}
