<?php
require_once('workoutplanpolynom.class.php');

class WorkoutInterval
{
	private int $runtime;
	private int $workout;
	public function __construct(int $runtime, int $workout)
	{
		$this->runtime = $runtime;
		$this->workout = $workout;
	}
	public function getRuntime()
	{
		return $this->runtime;
	}
	public function getWorkout()
	{
		return $this->workout;
	}
}

class WorkoutDefinition
{
	private \DateTime $startDate;
	private string $description;
	private WorkoutInterval $interval;
	private int|null $workoutFixed;
	private WorkoutPlanPolynom|null $workoutCalculated;

	public function __construct(
		\DateTime $startDate,
		string $description,
		WorkoutInterval $interval,
		int|null $workoutFixed,
		WorkoutPlanPolynom|null $workoutCalculated,
	) {
		$this->startDate = $startDate;
		$this->description = $description;
		$this->interval = $interval;
		$this->workoutFixed = $workoutFixed;
		$this->workoutCalculated = $workoutCalculated;
	}

	public function getStartDate()
	{
		return $this->startDate;
	}
	public function getDescription()
	{
		return $this->description;
	}
	public function calculateFor(\DateTime $date)
	{
		$dayno = $date->diff($this->startDate)->days % $this->interval->getRuntime();
		$ret = "";
		if ($dayno < $this->interval->getWorkout()) {
			if ($this->workoutFixed) {
				$ret .= $this->workoutFixed . " ";
			} else if ($this->workoutCalculated) {
				$dayForCalculated = $date->diff($this->startDate)->days;
				$ret .= $this->workoutCalculated->calc($dayForCalculated) . " ";
				if ($this->workoutCalculated->isPastEndDay($dayForCalculated)) {
					$ret .= "(Plan beendet.) ";
				}
			} else {
				$ret .= "!!Kein Wert!! ";
			}
			$ret .= $this->description;
		}
		return $ret;
	}
}
