<?php

namespace apiFuncs {

  function xsget($url)
  {
    $fake_user_agent = "Mozilla/5.0 (Windows NT 10.2; Win64; x64) AppleWebKit/666.66 (KHTML, like Gecko) Chrome/66.6.6666.66 Safari/666.66";
    \ini_set('user_agent', $fake_user_agent);
    $ret = \file_get_contents($url);
    if ($ret === false) { // error retrieving $url
      throw new \Exception("Error contacting $url");
    }
    return $ret;
  }

  function cache_contents($url, $file, $maxAge, $statusInHeader = false)
  {
    if (\file_exists($file) && (\time() - \filemtime($file) <= $maxAge)) {
      // cache is still fresh
      if ($statusInHeader) {
        header('X-CacheStatus: cached');
      } else {
        echo "<!--Cache status: cached-->";
      }
      return \file_get_contents($file);
    } else {
      // no cache, create one
      $cache = xsget($url, true);
      \file_put_contents($file, $cache);
      if ($statusInHeader) {
        header('X-CacheStatus: fresh');
      } else {
        echo "<!--Cache status: fresh-->";
      }
      return $cache;
    }
  }

  function formatAsDay($dayNum)
  {
    switch ($dayNum) {
      case 1:
        return 'Mo';
      case 2:
        return 'Di';
      case 3:
        return 'Mi';
      case 4:
        return 'Do';
      case 5:
        return 'Fr';
      case 6:
        return 'Sa';
      case 7:
        return 'So';
      default:
        return '??';
    }
  }

  function get($key, $default_value = '')
  {
    return isset($_GET[$key]) ? $_GET[$key] : $default_value;
  }

  function post($key, $default_value = '')
  {
    return isset($_POST[$key]) ? $_POST[$key] : $default_value;
  }

  function getpost($key, $default_value = '')
  {
    $ret = get($key, $default_value);
    if ($ret == $default_value) {
      $ret = post($key, $default_value);
    }
    return $ret;
  }

  function getNumParam($key, $default_value = 0, $maxVal = 255, $minVal = 0)
  {
    $param = \trim(getpost($key));
    if ($param != "") {
      if (\preg_match("#^([0-9]*)$#", $param, $match) && $match[0] !== '') {
        $paramInt = \intval($match[0]);
        if (($paramInt >= $minVal) && ($paramInt <= $maxVal)) {
          return $paramInt;
        }
      }
    }
    return $default_value;
  }
}
